#-----------------------------------------------------------------------------
#
#   Supersymmetry - Particles
#
#-----------------------------------------------------------------------------

from albow.controls import Label, Button
from albow.layout import Column
from albow.dialogs import Dialog, alert
from albow.resource import get_font

class DiscoveryDialog(Dialog):

	font1 = get_font(20, "VeraBd.ttf")
	font2 = get_font(14, "Vera.ttf")
	font3 = get_font(16, "VeraBd.ttf")

	def __init__(self, particle):
		Dialog.__init__(self)
		text = Column([
			Label("Congratulations!", font = self.font1),
			Label("You've found the", font = self.font2),
			Label(particle.name.upper(), font = self.font3),
		])
		contents = Column([
			text,
			Button("Hooray", action = self.ok),
		], align = 'r', spacing = 20, margin = 20)
		self.add(contents)
		self.shrink_wrap()

#-----------------------------------------------------------------------------

class Particle(object):

	discovery_chance = 0.01 # per tick

	def __init__(self, name, is_real = True, comment = None):
		self.name = name
		self.is_real = is_real
		self.comment = comment
	
	def announce_discovery(self):
		dlog = DiscoveryDialog(self)
		dlog.present()
		if self.comment:
			alert(self.comment)

#-----------------------------------------------------------------------------

def get(i):
	return particle_list[i]

def num_particles():
	#return 1 ###
	return len(particle_list)

particle_list = [
	Particle("selectron"),
	Particle("photino"),
	Particle("sneutrino"),
	Particle("arduino", False, "Oops, no, the Arduino is a computer, not a particle. Sorry. Carry on."),
	Particle("smuon"),
	Particle("gluino"),
	Particle("stauon"),
	Particle("bambino", False, "Actually, that's a Japanese cooking manga. Someone is playing silly buggers with the particle list."),
	Particle("down squark"),
	Particle("up squark"),
	Particle("wino", True, "That's not a joke, by the way. The wino is really a particle."),
	Particle("strange squark"),
	Particle("gravitino", True, "Warning: Excessive gravitino concentration from here on could form a quantum black hole and swallow the earth. Proceed with caution."),
	Particle("charm squark"),
	Particle("bottom squark"),
	Particle("zino"),
	Particle("top squark"),
]
